@@ -17,8 +17,6 @@
 #ifndef GRPC_SRC_CORE_LOAD_BALANCING_LB_POLICY_H
 #define GRPC_SRC_CORE_LOAD_BALANCING_LB_POLICY_H
 
-#include <grpc/support/port_platform.h>
-
 #include <stddef.h>
 #include <stdint.h>
 
@@ -37,8 +35,8 @@
 #include <grpc/event_engine/event_engine.h>
 #include <grpc/grpc.h>
 #include <grpc/impl/connectivity_state.h>
+#include <grpc/support/port_platform.h>
 
-#include "src/core/load_balancing/backend_metric_data.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/debug/trace.h"
 #include "src/core/lib/gprpp/debug_location.h"
@@ -50,13 +48,13 @@
 #include "src/core/lib/gprpp/work_serializer.h"
 #include "src/core/lib/iomgr/iomgr_fwd.h"
 #include "src/core/lib/iomgr/resolved_address.h"
+#include "src/core/load_balancing/backend_metric_data.h"
 #include "src/core/load_balancing/subchannel_interface.h"
 #include "src/core/resolver/endpoint_addresses.h"
+#include "src/core/telemetry/metrics.h"
 
 namespace grpc_core {
 
-extern DebugOnlyTraceFlag grpc_trace_lb_policy_refcount;
-
 /// Interface for load balancing policies.
 ///
 /// The following concepts are used here:
@@ -274,7 +272,8 @@ class LoadBalancingPolicy : public InternallyRefCounted<LoadBalancingPolicy> {
 
     virtual PickResult Pick(PickArgs args) = 0;
 
-    void Orphan() override {}
+   protected:
+    void Orphaned() override {}
   };
 
   /// A proxy object implemented by the client channel and used by the
@@ -299,6 +298,9 @@ class LoadBalancingPolicy : public InternallyRefCounted<LoadBalancingPolicy> {
     /// Requests that the resolver re-resolve.
     virtual void RequestReresolution() = 0;
 
+    /// Returns the channel target.
+    virtual absl::string_view GetTarget() = 0;
+
     /// Returns the channel authority.
     virtual absl::string_view GetAuthority() = 0;
 
@@ -319,6 +321,10 @@ class LoadBalancingPolicy : public InternallyRefCounted<LoadBalancingPolicy> {
     /// Returns the EventEngine to use for timers and async work.
     virtual grpc_event_engine::experimental::EventEngine* GetEventEngine() = 0;
 
+    /// Returns the stats plugin group for reporting metrics.
+    virtual GlobalStatsPluginRegistry::StatsPluginGroup&
+    GetStatsPluginGroup() = 0;
+
     /// Adds a trace message associated with the channel.
     enum TraceSeverity { TRACE_INFO, TRACE_WARNING, TRACE_ERROR };
     virtual void AddTraceEvent(TraceSeverity severity,
@@ -421,7 +427,7 @@ class LoadBalancingPolicy : public InternallyRefCounted<LoadBalancingPolicy> {
   // A picker that returns PickResult::Queue for all picks.
   // Also calls the parent LB policy's ExitIdleLocked() method when the
   // first pick is seen.
-  class QueuePicker : public SubchannelPicker {
+  class QueuePicker final : public SubchannelPicker {
    public:
     explicit QueuePicker(RefCountedPtr<LoadBalancingPolicy> parent)
         : parent_(std::move(parent)) {}
@@ -436,7 +442,7 @@ class LoadBalancingPolicy : public InternallyRefCounted<LoadBalancingPolicy> {
   };
 
   // A picker that returns PickResult::Fail for all picks.
-  class TransientFailurePicker : public SubchannelPicker {
+  class TransientFailurePicker final : public SubchannelPicker {
    public:
     explicit TransientFailurePicker(absl::Status status) : status_(status) {}
 
@@ -448,6 +454,19 @@ class LoadBalancingPolicy : public InternallyRefCounted<LoadBalancingPolicy> {
     absl::Status status_;
   };
 
+  // A picker that returns PickResult::Drop for all picks.
+  class DropPicker final : public SubchannelPicker {
+   public:
+    explicit DropPicker(absl::Status status) : status_(status) {}
+
+    PickResult Pick(PickArgs /*args*/) override {
+      return PickResult::Drop(status_);
+    }
+
+   private:
+    absl::Status status_;
+  };
+
  protected:
   std::shared_ptr<WorkSerializer> work_serializer() const {
     return work_serializer_;
@@ -476,6 +495,11 @@ class LoadBalancingPolicy : public InternallyRefCounted<LoadBalancingPolicy> {
   ChannelArgs channel_args_;
 };
 
+template <>
+struct ArenaContextType<LoadBalancingPolicy::SubchannelCallTrackerInterface> {
+  static void Destroy(LoadBalancingPolicy::SubchannelCallTrackerInterface*) {}
+};
+
 }  // namespace grpc_core
 
 #endif  // GRPC_SRC_CORE_LOAD_BALANCING_LB_POLICY_H
